unit MLRegWizard;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Basic_Wizard, ComCtrls, StdCtrls, ExtCtrls, Buttons, StatTools,
  MtxVec, TeeProcs, TeEngine, Chart, TeeComma, MtxBaseComp;

type
  TfrmMulLinRegWiz = class(TfrmBasicWizard)
    tsInputData: TTabSheet;
    tsReportConfig: TTabSheet;
    tsChart: TTabSheet;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    BitBtnA: TBitBtn;
    BitBtnY: TBitBtn;
    BitBtnW: TBitBtn;
    Edit1: TEdit;
    Edit2: TEdit;
    GroupBox2: TGroupBox;
    Label6: TLabel;
    Label7: TLabel;
    ChkBoxUseWeights: TCheckBox;
    ChkBoxRemoveInt: TCheckBox;
    Edit4: TEdit;
    Edit3: TEdit;
    MtxAnova: TMtxAnova;
    MtxMulLinReg: TMtxMulLinReg;
    GroupBox3: TGroupBox;
    ChkBoxCorr: TCheckBox;
    ChkBoxDescStat: TCheckBox;
    ChkBoxRegCoeff: TCheckBox;
    ChkBoxRes: TCheckBox;
    ChkBoxRegStats: TCheckBox;
    ChkBoxSerCorr: TCheckBox;
    TeeCommander1: TTeeCommander;
    Chart1: TChart;
    tsTextReport: TTabSheet;
    RichEdit: TRichEdit;
    procedure btnNextClick(Sender: TObject);
    procedure btnBackClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BitBtnAClick(Sender: TObject);
    procedure ChkBoxUseWeightsClick(Sender: TObject);
    procedure ChkBoxRemoveIntClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Edit4Change(Sender: TObject);
    procedure Edit1Change(Sender: TObject);
    procedure Edit3Change(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    tmpMtx: TMtx;
    tmpVec: TVec;
    ChartIndex : Integer;
    procedure TextReport;
    procedure ChartReport(AIndex: Integer);
    procedure CorrMtxReport;
    procedure DescStatReport;
    procedure RegCoeffReport;
    procedure ResidualReport;
    procedure BasicRegStatReport;
    procedure SerCorrReport;
  protected
    procedure RefreshButtons; override;
  public
    { Public declarations }
  end;

var
  frmMulLinRegWiz: TfrmMulLinRegWiz;

implementation

Uses MtxVecEdit, Math387, Statistics, StatChar, Series,
     MtxVecTee, StatSeries, ErrorBar, Probabilities;

{ page numbers }
const
  pgInputData    = 0; { input data}
  pgReportConfig = 1; { config reports }
  pgTextReport = 2;   { textual part }
  pgChartReport0 = 3; { histogram }
  pgChartReport1 = 4; { probability plot }
  pgChartReport2 = 5; { Serial correlation }
  pgChartReport3 = 6; { residuals vs predicted }

{$R *.DFM}

{ TfrmBasicWizard1 }

procedure TfrmMulLinRegWiz.BasicRegStatReport;
begin
  RichEdit.Paragraph.TabCount := 1;
  RichEdit.Paragraph.Tab[0] := 100;
  RichEdit.SelAttributes.Style := [fsBold,fsUnderline];
  RichEdit.Lines.Add('Basic regression statistics');
  With RichEdit.Lines do
  begin
    Add('Residuals variance : ' + chr(9) + FormatFloat(FormatString,MtxMulLinReg.RegressStatistics.ResidualVar));
    Add('R squared : '+ chr(9) + FormatFloat(FormatString,MtxMulLinReg.RegressStatistics.R2));
    Add('Adjusted R squared :'+ chr(9) + FormatFloat(FormatString,MtxMulLinReg.RegressStatistics.AdjustedR2));
    Add('F value : '+ chr(9) + FormatFloat(FormatString,MtxMulLinReg.RegressStatistics.F));
    Add('Probability of F : '+ chr(9) + FormatFloat(FormatString,MtxMulLinReg.RegressStatistics.SignifProb));
    Add(Chr(13));
  end;
end;

procedure TfrmMulLinRegWiz.ChartReport(AIndex: Integer);
var tmpVec1,tmpVec2: TVec;
    minX,MaxX,minY,MaxY: TSample;
begin
  Chart1.FreeAllSeries;
  if (AIndex = pgChartReport0) then
  begin
    { setup chart title and axes }
    Chart1.Title.Text.Clear;
    Chart1.Title.Text.Add('Histogram of residuals');
    Chart1.LeftAxis.Title.Caption := 'Count';
    Chart1.BottomAxis.Title.Caption := 'Residuals';
    Chart1.LeftAxis.AutomaticMinimum := false;
    Chart1.LeftAxis.Minimum := 0.0;
    { add histogram series }
    Chart1.AddSeries(THistogramSeries.Create(Self));
    Chart1.Series[0].SeriesColor := clBlue;
    THistogramSeries(Chart1.Series[0]).LinesPen.Visible := true;
    { draw histogram }
    tmpVec1 := TVec.Create;
    tmpVec2 := TVec.Create;
    try
      Histogram(MtxMulLinReg.RegressResult.Residuals,10,tmpVec1,tmpVec2,true);
      DrawValues(tmpVec2,tmpVec1,Chart1.Series[0]);
    finally
      tmpVec1.Destroy;
      tmpVec2.Destroy;
    end;
  end
  else if (AIndex = pgChartReport1) then
  begin
    { setup chart title and axes }
    Chart1.Title.Text.Clear;
    Chart1.AddSeries(TStatProbSeries.Create(Self));
    Chart1.Series[0].SeriesColor := clBlue;
    TStatProbSeries(Chart1.Series[0]).Pointer.Style := psCircle;
    Chart1.Title.Text.Add('Residuals normal probability plot');
    Chart1.LeftAxis.Title.Caption := 'Expected normals';
    Chart1.LeftAxis.Automatic := true;
    Chart1.BottomAxis.Title.Caption := 'Residuals';
    tmpVec1 := TVec.Create;
    tmpVec2 := TVec.Create;
    try
      StatNormalPlot(MtxMulLinReg.RegressResult.Residuals,tmpVec1,tmpVec2,MinX,MaxX,MinY,MaxY,False);
      TStatProbSeries(Chart1.Series[0]).MinX:= MinX;
      TStatProbSeries(Chart1.Series[0]).MaxX:= MaxX;
      TStatProbSeries(Chart1.Series[0]).MinY:= MinY;
      TStatProbSeries(Chart1.Series[0]).MaxY:= MaxY;
      DrawValues(tmpVec1,tmpVec2,Chart1.Series[0]);
    finally
      tmpVec1.Destroy;
      tmpVec2.Destroy;
    end;
  end
  else if (AIndex = pgChartReport2) then
  begin
    { setup chart title and axes }
    Chart1.Title.Text.Clear;
    Chart1.Title.Text.Add('Residuals Serial Correlation');
    Chart1.LeftAxis.Title.Caption := 'Residuals';
    Chart1.LeftAxis.Automatic := true;
    Chart1.BottomAxis.Title.Caption := 'Residuals Lagged Once';
    Chart1.AddSeries(TPointSeries.Create(Self));
    Chart1.Series[0].SeriesColor := clBlue;
    TPointSeries(Chart1.Series[0]).Pointer.Style := psCircle;
    TPointSeries(Chart1.Series[0]).Pointer.InflateMargins := true;
    tmpVec1 := TVec.Create;
    tmpVec2 := TVec.Create;
    try
      With MtxMulLinReg.RegressResult do
      begin
        tmpVec1.Size(Residuals.Length -1);
        tmpVec2.Size(Residuals.Length -1);
        tmpVec1.Copy(Residuals,1,0,tmpVec1.Length);
        tmpVec2.Copy(Residuals,0,0,tmpVec2.Length);
        DrawValues(tmpVec2,tmpVec1,Chart1.Series[0]);
      end;
    finally
      tmpVec1.Destroy;
      tmpVec2.Destroy;
    end;
  end
  else if (AIndex = pgChartReport3) then
  begin
    { setup chart title and axes }
    Chart1.Title.Text.Clear;
    Chart1.Title.Text.Add('Residuals versus predicted values');
    Chart1.LeftAxis.Title.Caption := 'Residuals';
    Chart1.BottomAxis.Title.Caption := 'Predicted';
    Chart1.LeftAxis.Automatic := true;
    { add point series }
    Chart1.AddSeries(TPointSeries.Create(Self));
    Chart1.Series[0].SeriesColor := clBlue;
    TPointSeries(Chart1.Series[0]).Pointer.Style := psCircle;
    TPointSeries(Chart1.Series[0]).Pointer.InflateMargins := true;
    With MtxMulLinReg.RegressResult do
      DrawValues(YCalc,Residuals,Chart1.Series[0]);
  end;
end;

procedure TfrmMulLinRegWiz.CorrMtxReport;
var AdjCorrMtx: TMtx;
begin
  RichEdit.SelAttributes.Style := [fsBold,fsUnderline];
  RichEdit.Lines.Add('Correlation matrix');
  AdjCorrMtx := TMtx.Create;
  with MtxMulLinReg do
  try
    AdjCorrMtx.Size(A.Rows,A.Cols + 1);
    RichEdit.Paragraph.TabCount := AdjCorrMtx.Cols;
    SetupTabs(RichEdit.Paragraph.TabCount, RichEdit);
    AdjCorrMtx.Copy(A,0,0,0,0,A.Rows,A.Cols,false);
    AdjCorrMtx.SetCol(Y,A.Cols);
    CorrCoef(AdjCorrMtx,tmpMtx);
    tmpMtx.ValuesToStrings(RichEdit.Lines,Chr(9),FormatString);
  finally
    AdjCorrMtx.Destroy;
  end;
  RichEdit.Lines.Add(Chr(13));
end;

procedure TfrmMulLinRegWiz.DescStatReport;
var i: Integer;
  tmpMin,tmpMean,
  tmpStdDev,tmpMax: TSample;
  LineText : String;
begin
  RichEdit.Paragraph.TabCount := 5;
  SetupTabs(RichEdit.Paragraph.TabCount, RichEdit);
  RichEdit.SelAttributes.Style := [fsBold,fsUnderline];
  RichEdit.Lines.Add('Descriptive Statistics');
  RichEdit.SelAttributes.Color := clBlue;
  RichEdit.Lines.Add('Variable'+chr(9)+'Mean'+chr(9)+'Std.Dev.'
                      +chr(9)+'Min'+chr(9)+'Max');
  { basic descriptive statistics }
  for i := 0 to MtxMulLinReg.A.Cols-1 do
  begin
    tmpVec.GetCol(MtxMulLinReg.A,0,i,MtxMulLinReg.A.Rows);
    tmpMean := tmpVec.Mean;
    tmpStdDev := tmpVec.StdDev(tmpMean);
    tmpMax := tmpVec.Max;
    tmpMin := tmpVec.Min;
    LineText :='A column '+IntToStr(i)+Chr(9);
    LineText := LineText + FormatFloat(FormatString,tmpMean)+Chr(9);
    LineText := LineText + FormatFloat(FormatString,tmpStdDev) + Chr(9);
    LineText := LineText + FormatFloat(FormatString,tmpMin) + Chr(9);
    LineText := LineText + FormatFloat(FormatString,tmpMax);
    RichEdit.Lines.Add(LineText);
  end;
  tmpMean := MtxMulLinReg.Y.Mean;
  tmpStdDev := MtxMulLinReg.Y.StdDev(tmpMean);
  tmpMax := MtxMulLinReg.Y.Max;
  tmpMin := MtxMulLinReg.Y.Min;
  LineText := 'Y column'+chr(9);
  LineText := LineText + FormatFloat(FormatString,tmpMean) + Chr(9);
  LineText := LineText + FormatFloat(FormatString,tmpStdDev) + Chr(9);
  LineText := LineText + FormatFloat(FormatString,tmpMin) + Chr(9);
  LineText := LineText + FormatFloat(FormatString,tmpMax);
  RichEdit.Lines.Add(LineText);
  RichEdit.Lines.Add(Chr(13));
end;

procedure TfrmMulLinRegWiz.RefreshButtons;
begin
  inherited;
  With BtnNext do
  begin
    Case PageControl.ActivePage.PageIndex of
      pgInputData : Enabled := MtxMulLinReg.ValidSystem;
      pgChartReport0: Enabled := (ChartIndex >= pgChartReport0) and (ChartIndex < pgChartReport3);
    else
      Enabled := true;
    end;
    if PageControl.ActivePage.PageIndex=pgReportConfig then
       Caption := '&Calculate'
    else
       Caption := 'Next >';
  end;
end;

procedure TfrmMulLinRegWiz.RegCoeffReport;
var i: Integer;
  LineText : String;
  CIText : String;
  var TVals, PVals: TVec;
begin
  CreateIt(TVals,PVals);
  { calculate t values }
  { This is the t-test value for testing the hypothesis that j = 0 versus 
    the alternative that after removing the influence of all other X's. 
    This t-value has Rows-Cols-1 degrees of freedom.
  }
  TVals.DivideVec(MtxMulLinReg.RegressResult.B,MtxMulLinReg.RegressResult.BStdDev);
  
  { calculate p values }
  { The p-value is the probability that above T-statistic will take on a value 
    at least as extreme as the actually observed value, assuming that the null hypothesis
    is true (i.e., the regression estimate is equal to zero). If the p-value is less than
    alpha, say 0.05, the null hypothesis is rejected. 
    This p-value is for a two-tail test.
  }
  StudentCDF(TVals,MtxMulLinReg.A.Rows - MtxMulLinReg.A.Cols -1,PVals);
  for i := 0 to PVals.Length -1 do
    PVals.Values[i] := 2.0*Min(PVals.Values[i],1.0 - PVals.Values[i]);

  try
    CIText := FormatFloat('0.00 %',100*(1-MtxMulLinReg.Alpha));
    With RichEdit do
    begin
      Paragraph.TabCount := 7;
      SetupTabs(RichEdit.Paragraph.TabCount, RichEdit);
      SelAttributes.Style := [fsBold,fsUnderline];
      Lines.Add('Regression coefficients');
      SelAttributes.Color := clBlue;
      Lines.Add('Independent'+chr(9)+'Regression'+chr(9)+'Standard'
                      +chr(9)+'T Value'+chr(9)+'P Value'+chr(9)+'Lower'+chr(9)+'Upper');
      SelAttributes.Color := clBlue;
      Lines.Add('variable'+chr(9)+'coefficient'+chr(9)+'deviation'+chr(9)+'Ho: B=0'+
                      chr(9)+chr(9)+CIText+' CI'+chr(9)+CIText+' CI');
      { regression coefficients }
      for i := 0 to MtxMulLinReg.RegressResult.B.Length -1 do
      with MtxMulLinReg.RegressResult do
      begin
        LineText := 'Variable ' + IntToStr(i)+chr(9);
        LineText := LineText + FormatFloat(FormatString,B.Values[i])+chr(9);
        LineText := LineText + FormatFloat(FormatString,BStdDev.Values[i])+chr(9);
        LineText := LineText + FormatFloat(FormatString,TVals.Values[i])+chr(9);
        LineText := LineText + FormatFloat(FormatString,PVals.Values[i])+chr(9);
        LineText := LineText + FormatFloat(FormatString,BConfInt.Values[i,0])+chr(9);
        LineText := LineText + FormatFloat(FormatString,BConfInt.Values[i,1]);
        Lines.Add(LineText)
      end;
      Lines.Add(Chr(13));
    end;
  finally
    FreeIt(TVals,PVals);
  end;
end;

procedure TfrmMulLinRegWiz.ResidualReport;
var LineText : String;
    i : Integer;
begin
  With RichEdit do
  begin
    Paragraph.TabCount := 3;
    SetupTabs(RichEdit.Paragraph.TabCount, RichEdit);
    SelAttributes.Style := [fsBold,fsUnderline];
    Lines.Add('Residuals');
    SelAttributes.Color := clBlue;
    Lines.Add('Actual'+chr(9)+'Predicted'+chr(9)+'Residual');
  { regression coefficients }
    for i := 0 to MtxMulLinReg.Y.Length -1 do
    with MtxMulLinReg do
    begin
      LineText := FormatFloat(FormatString,Y.Values[i])+chr(9);
      LineText := LineText + FormatFloat(FormatString,RegressResult.YCalc.Values[i])+chr(9);
      LineText := LineText + FormatFloat(FormatString,RegressResult.Residuals.Values[i]);
      Lines.Add(LineText)
    end;
    Lines.Add(Chr(13));
  end;
end;


procedure TfrmMulLinRegWiz.SerCorrReport;
var SS : TSample;
    Lags : Integer;
    i : Integer;
    LineText : String;
begin
  RichEdit.Paragraph.TabCount := 2;
  SetupTabs(RichEdit.Paragraph.TabCount, RichEdit);
  RichEdit.SelAttributes.Style := [fsBold,fsUnderline];
  RichEdit.Lines.Add('Serial correlation (up to 10th lag)');
  RichEdit.SelAttributes.Color := clBlue;
  RichEdit.Lines.Add('Lag'+chr(9)+'Correlation');
  With MtxMulLinReg.RegressResult do
  begin
    Lags := Min(Residuals.Length,11); { max 10 lags }
    SS := Sqr(Residuals.NormL2); { sum of squares }
    tmpVec.AutoCorrNormal(Residuals,Lags);
  end;
  tmpVec.Scale(1.0/SS);
  { regression coefficients }
  for i := 1 to tmpVec.Length -1 do
  begin
    LineText := IntToStr(i)+chr(9);
    LineText := LineText + FormatFloat(FormatString,tmpVec.Values[i]);
    RichEdit.Lines.Add(LineText)
  end;
  {RichEdit.Lines.Add('Serial correlations are significant if their '
                    +' absolute values are greater than : ' + FormatFloat(FormatString,Power(2,Sqrt(tmpVec.Length))));
  }
  if tmpVec.Length >=2 then
  begin
    RichEdit.Lines.Add(Chr(13));
    RichEdit.Lines.Add('Durbin-Watson value : ' + FormatFloat(FormatString,2.0*(1.0-tmpVec.Values[1])));
  end;
  RichEdit.Lines.Add(Chr(13));
end;

procedure TfrmMulLinRegWiz.TextReport;
begin
  RichEdit.Enabled := false;
  RichEdit.Lines.Clear;
  try
    RichEdit.DefAttributes.Color := clBlack;
    RichEdit.DefAttributes.Pitch := fpFixed;
    RichEdit.DefAttributes.Style := [];
    RichEdit.SelAttributes.Style := [fsBold,fsUnderline];
    RichEdit.SelAttributes.Size := 16;
    RichEdit.Lines.Add('Multiple linear regression');
    RichEdit.Lines.Add(Chr(13));
    RichEdit.Lines.Add(Chr(13));
    if ChkBoxRegCoeff.Checked then RegCoeffReport;
    if ChkBoxRes.Checked then ResidualReport;
    if ChkBoxRegStats.Checked then BasicRegStatReport;
    if ChkBoxDescStat.Checked then DescStatReport;
    if ChkBoxCorr.Checked then CorrMtxReport;
    if ChkBoxSerCorr.Checked then SerCorrReport;
  finally
    RichEdit.Enabled := true;
  end;
end;

procedure TfrmMulLinRegWiz.btnNextClick(Sender: TObject);
begin
  inherited;
  case PageControl.ActivePage.PageIndex of
    pgInputData : begin
                      PageControl.ActivePage := PageControl.Pages[pgReportConfig];
                      lblHeader.Caption := 'Step 2 : Define regression reports';
                  end;
    pgReportConfig: begin
                      MtxMulLinReg.Recalc;
                      PageControl.ActivePage := PageControl.Pages[pgTextReport];
                      lblHeader.Caption := 'Regression results : text report';
                      TextReport;
                    end;
    pgTextReport :  begin
                      lblHeader.Caption := 'Regression results : Charts';
                      PageControl.ActivePage := PageControl.Pages[pgChartReport0];
                      ChartIndex := pgChartReport0;
                      ChartReport(ChartIndex);
                    end;
    pgChartReport0: begin
                      ChartIndex := ChartIndex + 1;
                      lblHeader.Caption := 'Regression results : Charts';
                      ChartReport(ChartIndex);
                    end;
  end;
  RefreshButtons;
end;

procedure TfrmMulLinRegWiz.btnBackClick(Sender: TObject);
begin
  inherited;
  case PageControl.ActivePage.PageIndex of
    pgReportConfig :  begin
                        PageControl.ActivePage := PageControl.Pages[pgInputData];
                        lblHeader.Caption := 'Step 1 : Define system of linear equations';
                      end;

    pgTextReport :  begin
                      PageControl.ActivePage := PageControl.Pages[pgReportConfig];
                      lblHeader.Caption := 'Step 2 : Define regression reports';
                    end;
    pgChartReport0: begin
                      ChartIndex := ChartIndex - 1;
                      if ChartIndex < pgChartReport0 then
                      begin
                        PageControl.ActivePage := PageControl.Pages[pgTextReport];
                        lblHeader.Caption := 'Step 2 : Define regression reports';
                      end else ChartReport(ChartIndex);
                    end;
  end;
  RefreshButtons;
end;

procedure TfrmMulLinRegWiz.FormCreate(Sender: TObject);
begin
  inherited;
  tmpMtx := TMtx.Create;
  tmpVec := TVec.Create;
  with TeeCommander1 do
  begin
    ButtonSave.Visible := false;
    ButtonPrint.Visible := false;
  end;
end;

procedure TfrmMulLinRegWiz.BitBtnAClick(Sender: TObject);
begin
  if Sender = BitBtnA then
  begin
    ViewValues(MtxMulLinReg.A,'Independent variables',true,true);
    Edit1.Text := IntToStr(MtxMulLinReg.A.Rows);
    Edit2.Text := IntToStr(MtxMulLinReg.A.Cols);
    MtxMulLinReg.Y.Length := MtxMulLinReg.A.Rows;
    MtxMulLinReg.Weights.Length := MtxMulLinReg.A.Rows;
  end
  else if Sender = BitBtnY then ViewValues(MtxMulLinReg.Y,'Dependent variable',true,true)
  else if Sender = BitBtnW then ViewValues(MtxMulLinReg.Weights,'Weights',true,true);
  RefreshButtons;
end;

procedure TfrmMulLinRegWiz.ChkBoxUseWeightsClick(Sender: TObject);
begin
  MtxMulLinReg.UseWeights := ChkBoxUseWeights.Checked;
  RefreshButtons;
end;

procedure TfrmMulLinRegWiz.ChkBoxRemoveIntClick(Sender: TObject);
begin
  MtxMulLinReg.Constant := Not (ChkBoxRemoveInt.Checked);
end;

procedure TfrmMulLinRegWiz.FormDestroy(Sender: TObject);
begin
  tmpMtx.Destroy;
  tmpVec.Destroy;
  inherited;
end;

procedure TfrmMulLinRegWiz.Edit4Change(Sender: TObject);
begin
  FormatString := Edit4.Text;
  try
    FormatFloat(FormatString,1.22);  { dummy test }
  except
  end;
end;

procedure TfrmMulLinRegWiz.Edit1Change(Sender: TObject);
var i : Integer;
begin
  try
    i := StrToInt(TEdit(Sender).Text);
    if Sender = Edit1 then
    begin
      MtxMulLinReg.A.Rows := i;
      MtxMulLinReg.Y.Length := i;
      MtxMulLinReg.Weights.Length := i;
    end
    else if Sender = Edit2 then MtxMulLinReg.A.Cols := i;
  except
  end;
  RefreshButtons;
end;

procedure TfrmMulLinRegWiz.Edit3Change(Sender: TObject);
begin
  try
    MtxMulLinReg.Alpha := StrToFloat(Edit3.Text);
  except
  end;
end;

procedure TfrmMulLinRegWiz.FormShow(Sender: TObject);
begin
  inherited;
  { initialize page }
  PageControl.ActivePage := tsInputData;
  lblHeader.Caption := 'Step 1 : Define system of linear equations';
  RefreshButtons;
  Edit1.Text := IntToStr(MtxMulLinReg.A.Rows);
  Edit2.Text := IntToStr(MtxMulLinReg.A.Cols);
  Edit3.Text := FormatFloat('0.00',MtxMulLinReg.Alpha);
  Edit4.Text := FormatString;
  FormatString := Edit4.Text;
end;

end.
 