unit PCAWizard;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Basic_Wizard, ComCtrls, StdCtrls, ExtCtrls, StatTools,
  Math387, MtxVec, TeeComma, TeeProcs, TeEngine, Chart, MtxBaseComp;

type
  TfrmPCAWiz = class(TfrmBasicWizard)
    tsInputData: TTabSheet;
    MtxPCA: TMtxPCA;
    tsReportConfig: TTabSheet;
    GroupBox1: TGroupBox;
    tsTextReport: TTabSheet;
    RichEdit: TRichEdit;
    ChkBoxDescStat: TCheckBox;
    ChkBoxCorrMtx: TCheckBox;
    ChkBoxEigValues: TCheckBox;
    GroupBox2: TGroupBox;
    Label4: TLabel;
    EditFmtString: TEdit;
    GroupBox3: TGroupBox;
    Label1: TLabel;
    EditCols: TEdit;
    Label2: TLabel;
    EditRows: TEdit;
    Label3: TLabel;
    BtnEditData: TButton;
    Label5: TLabel;
    CBoxPCAMode: TComboBox;
    ChkBoxPrinComp: TCheckBox;
    ChkBoxZScores: TCheckBox;
    tsChart: TTabSheet;
    Chart: TChart;
    TeeCommander1: TTeeCommander;
    ChkBoxBarlett: TCheckBox;
    procedure BtnEditDataClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure EditColsChange(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure btnBackClick(Sender: TObject);
    procedure EditFmtStringChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure CBoxPCAModeChange(Sender: TObject);
  private
    { Private declarations }
    tmpVec: TVec;
    tmpMtx: TMtx;
    ChartIndex : Integer;
    procedure TextReport;
    procedure DescStatReport;
    procedure CorrMtxReport;
    procedure EigValuesReport;
    procedure PrinCompReport;
    procedure ZScoresReport;
    procedure BarlettReport;
    procedure ChartReport(AIndex: Integer);
  protected
    procedure RefreshButtons; override;
  public
    { Public declarations }
  end;

var
  frmPCAWiz: TfrmPCAWiz;

implementation

Uses MtxVecTee, MtxVecEdit, Statistics, Series, TeeBoxPlot;

Const
  pgInputData    = 0; { input data}
  pgReportConfig = 1; { config reports }
  pgTextReport = 2;   { textual part }
  pgChartReport0 = 3; { Box plot }
  pgChartReport1 = 4; { "Scree plot" }

{$R *.DFM}

procedure TfrmPCAWiz.BtnEditDataClick(Sender: TObject);
begin
  ViewValues(MtxPCA.Data,'Editinig data...',true,true);
  EditCols.Text := IntToStr(MtxPCA.Data.Cols);
  EditRows.Text := IntToStr(MtxPCA.Data.Rows);
  RefreshButtons;
end;

procedure TfrmPCAWiz.RefreshButtons;
begin
  inherited;
  With BtnNext do
  begin
    Case PageControl.ActivePage.PageIndex of
      pgInputData : Enabled := (MtxPCA.Data.Rows * MtxPCA.Data.Cols > 0);
      pgChartReport0: Enabled := ChartIndex < pgChartReport1;
    else
      Enabled := true;
    end;
    if PageControl.ActivePage.PageIndex=pgReportConfig then
       Caption := '&Calculate'
    else
       Caption := 'Next >';
  end;
end;

procedure TfrmPCAWiz.FormShow(Sender: TObject);
begin
  inherited;
  With TeeCommander1 do
  begin
    ButtonPrint.Visible := false;
    ButtonSave.Visible := false;
  end;
  { initialize page }
  PageControl.ActivePage := tsInputData;
  lblHeader.Caption := 'Step 1 : Define data';
  RefreshButtons;
  EditRows.Text := IntToStr(MtxPCA.Data.Rows);
  EditCols.Text := IntToStr(MtxPCA.Data.Cols);
  EditFmtString.Text := FormatString;
  CBoxPCAMode.ItemIndex := Ord(MtxPCA.PCAMode);
end;

procedure TfrmPCAWiz.EditColsChange(Sender: TObject);
var i : Integer;
begin
  try
    i := StrToInt(TEdit(Sender).Text);
    if Sender = EditCols then
      MtxPCA.Data.Cols := i
    else if Sender = EditRows then
      MtxPCA.Data.Rows := i;
  except
  end;
  RefreshButtons;
end;

procedure TfrmPCAWiz.btnNextClick(Sender: TObject);
begin
  inherited;
  case PageControl.ActivePage.PageIndex of
    pgInputData : begin
                    PageControl.ActivePage := PageControl.Pages[pgReportConfig];
                    lblHeader.Caption := 'Step 2 : Define PCA reports';
                  end;
    pgReportConfig: begin
                      MtxPCA.Recalc;
                      PageControl.ActivePage := PageControl.Pages[pgTextReport];
                      lblHeader.Caption := 'PCA : text reports';
                      TextReport;
                    end;
    pgTextReport :  begin
                      lblHeader.Caption := 'PCA : Charts';
                      PageControl.ActivePage := PageControl.Pages[pgChartReport0];
                      ChartIndex := pgChartReport0;
                      ChartReport(ChartIndex);
                    end;
    pgChartReport0: begin
                      ChartIndex := ChartIndex + 1;
                      lblHeader.Caption := 'PCA : Charts';
                      ChartReport(ChartIndex);
                    end;

  end;
  RefreshButtons;

end;

procedure TfrmPCAWiz.btnBackClick(Sender: TObject);
begin
  inherited;
  case PageControl.ActivePage.PageIndex of
    pgReportConfig :  begin
                        PageControl.ActivePage := PageControl.Pages[pgInputData];
                        lblHeader.Caption := 'Step 1 : Define data';
                      end;
    pgTextReport :  begin
                      PageControl.ActivePage := PageControl.Pages[pgReportConfig];
                      lblHeader.Caption := 'Step 2 : Define PCA reports';
                    end;
    pgChartReport0: begin
                      ChartIndex := ChartIndex - 1;
                      if ChartIndex < pgChartReport0 then
                      begin
                        PageControl.ActivePage := PageControl.Pages[pgTextReport];
                        lblHeader.Caption := 'Step 2 : Define PCA reports';
                      end else ChartReport(ChartIndex);
                    end;
  end;
  RefreshButtons;
end;

procedure TfrmPCAWiz.TextReport;
begin
  RichEdit.Enabled := false;
  RichEdit.Lines.Clear;
  try
    RichEdit.DefAttributes.Color := clBlack;
    RichEdit.DefAttributes.Pitch := fpFixed;
    RichEdit.DefAttributes.Style := [];
    RichEdit.SelAttributes.Style := [fsBold,fsUnderline];
    RichEdit.SelAttributes.Size := 16;
    RichEdit.Lines.Add('Principal Components Analysis');
    RichEdit.Lines.Add(Chr(13));
    RichEdit.Lines.Add(Chr(13));
    if ChkBoxDescStat.Checked then DescStatReport;
    if ChkBoxCorrMtx.Checked then CorrMtxReport;
    if ChkBoxBarlett.Checked then BarlettReport;
    if ChkBoxEigValues.Checked then EigValuesReport;
    if ChkBoxPrinComp.Checked then PrinCompReport;
    if ChkBoxZScores.Checked then ZScoresReport;
  finally
    RichEdit.Enabled := true;
  end;
end;

procedure TfrmPCAWiz.DescStatReport;
var i: Integer;
  tmpMin,tmpMean,
  tmpStdDev,tmpMax: TSample;
  LineText : String;
begin
  RichEdit.Paragraph.TabCount := 5;
  SetupTabs(RichEdit.Paragraph.TabCount, RichEdit);
  RichEdit.SelAttributes.Style := [fsBold,fsUnderline];
  RichEdit.Lines.Add('Descriptive Statistics');
  RichEdit.SelAttributes.Color := clBlue;
  RichEdit.Lines.Add('Variable'+chr(9)+'Mean'+chr(9)+'Std.Dev.'
                      +chr(9)+'Min'+chr(9)+'Max');
  { basic descriptive statistics }
  for i := 0 to MtxPCA.Data.Cols-1 do
  begin
    tmpVec.GetCol(MtxPCA.Data,0,i,MtxPCA.Data.Rows);
    tmpMean := tmpVec.Mean;
    tmpStdDev := tmpVec.StdDev(tmpMean);
    tmpMax := tmpVec.Max;
    tmpMin := tmpVec.Min;
    LineText :='A column '+IntToStr(i)+Chr(9);
    LineText := LineText + FormatFloat(FormatString,tmpMean)+Chr(9);
    LineText := LineText + FormatFloat(FormatString,tmpStdDev) + Chr(9);
    LineText := LineText + FormatFloat(FormatString,tmpMin) + Chr(9);
    LineText := LineText + FormatFloat(FormatString,tmpMax);
    RichEdit.Lines.Add(LineText);
  end;
  RichEdit.Lines.Add(Chr(13));
end;

procedure TfrmPCAWiz.EditFmtStringChange(Sender: TObject);
begin
  FormatString := EditFmtString.Text;
  try
    FormatFloat(FormatString,1.22);  { dummy test }
  except
  end;
end;

procedure TfrmPCAWiz.FormCreate(Sender: TObject);
begin
  inherited;
  tmpVec := TVec.Create;
  tmpMtx := TMtx.Create;
end;

procedure TfrmPCAWiz.FormDestroy(Sender: TObject);
begin
  tmpVec.Destroy;
  tmpMtx.Destroy;
  inherited;
end;

procedure TfrmPCAWiz.CorrMtxReport;
begin
  RichEdit.SelAttributes.Style := [fsBold,fsUnderline];
  RichEdit.Paragraph.TabCount := MTxPCA.Data.Cols;
  SetupTabs(RichEdit.Paragraph.TabCount, RichEdit);
  case MtxPCA.PCAMode of
    PCACovMat:  begin
                  RichEdit.Lines.Add('Covariance matrix');
                  Covariance(MtxPCA.Data,tmpMtx,false);
                end;
    PCACorrMat: begin
                  RichEdit.Lines.Add('Correlation matrix');
                  CorrCoef(MtxPCA.Data,tmpMtx);
                end;
  end;
  tmpMtx.ValuesToStrings(RichEdit.Lines,Chr(9),FormatString);
  RichEdit.Lines.Add(Chr(13));
  { Ln(R) }
  RichEdit.Lines.Add('Ln(R) = '+ FormatFloat(FormatString,Ln(tmpMtx.Determinant)));
  RichEdit.Lines.Add(Chr(13));
end;

procedure TfrmPCAWiz.EigValuesReport;
var LineText : String;
    i : Integer;
begin
  RichEdit.SelAttributes.Style := [fsBold,fsUnderline];
  RichEdit.Lines.Add('Eigenvalues');
  RichEdit.Paragraph.TabCount := 3;
  SetupTabs(RichEdit.Paragraph.TabCount, RichEdit);
  RichEdit.SelAttributes.Color := clBlue;
  RichEdit.Lines.Add('Variable'+chr(9)+'Eigenvalue'+chr(9)+'Percentage');
  for i := 0 to MtxPCA.EigValues.Length - 1 do
  begin
    LineText := IntToStr(i)+chr(9)+FormatFloat(FormatString,MtxPCA.EigValues.Values[i]);
    LineText := LineText + chr(9)+ FormatFloat(FormatString,MtxPCA.TotalVarPct.Values[i]);
    RichEdit.Lines.Add(LineText);
  end;
  RichEdit.Lines.Add(Chr(13));
end;

procedure TfrmPCAWiz.CBoxPCAModeChange(Sender: TObject);
begin
  MtxPCA.PCAMode := TPCAMode(CBoxPCAMode.ItemIndex);
end;

procedure TfrmPCAWiz.PrinCompReport;
begin
  RichEdit.SelAttributes.Style := [fsBold,fsUnderline];
  RichEdit.Lines.Add('Principal components (Eigenvectors)');
  RichEdit.Paragraph.TabCount := MtxPCA.Data.Cols;
  SetupTabs(RichEdit.Paragraph.TabCount, RichEdit);
  MtxPCA.PC.ValuesToStrings(RichEdit.Lines);
  RichEdit.Lines.Add(Chr(13));
end;

procedure TfrmPCAWiz.ZScoresReport;
begin
  RichEdit.SelAttributes.Style := [fsBold,fsUnderline];
  RichEdit.Lines.Add('Z Scores');
  RichEdit.Paragraph.TabCount := MtxPCA.ZScores.Cols;
  SetupTabs(RichEdit.Paragraph.TabCount, RichEdit);
  MtxPCA.ZScores.ValuesToStrings(RichEdit.Lines);
  RichEdit.Lines.Add(Chr(13));
end;

procedure TfrmPCAWiz.ChartReport(AIndex: Integer);
var i: Integer;
    tmpVec1: TVec;
begin
  Chart.FreeAllSeries;
  { Box plot }
  if (AIndex = pgChartReport0) then
  begin
    { setup chart title and axes }
    Chart.Title.Text.Clear;
    Chart.Legend.Visible := false;
    Chart.Title.Text.Add('Box plot');
    Chart.LeftAxis.Title.Caption := 'Values';
    Chart.LeftAxis.Automatic := true;
    Chart.BottomAxis.Title.Caption := 'Group (Column)';
    tmpVec1 := TVec.Create;
    try
      { add BoxPlot series }
      for i := 0 to MtxPCA.Data.Cols - 1 do
      begin
        Chart.AddSeries(TBoxSeries.Create(Chart));
        tmpVec1.GetCol(MtxPCA.Data,i);
        Chart.Series[i].Title := 'Column' + IntToStr(i);
        with TBoxSeries(Chart.Series[i]) do
        begin
          Position := i;
          Box.Brush.Color := clYellow;
          Pen.Color := clRed;
          SeriesColor := clRed;
        end;
        tmpVec1.SortAscend;
        DrawValues(tmpVec1,Chart.Series[i]);
      end;
    finally
      tmpVec1.Destroy;
    end;
  end else
  { Box plot }
  if (AIndex = pgChartReport1) then
  begin
    { setup chart title and axes }
    Chart.Title.Text.Clear;
    Chart.Legend.Visible := true;
    Chart.Title.Text.Add('Scree plot');
    Chart.LeftAxis.Title.Caption := 'Percentage';
    Chart.LeftAxis.Automatic := false;
    Chart.LeftAxis.SetMinMax(0,101);
    Chart.BottomAxis.Title.Caption := 'Eigenvalue';
    tmpVec1 := TVec.Create;
    try
      { add Line + bar series }
      Chart.AddSeries(TBarSeries.Create(Chart));
      Chart.AddSeries(TLineSeries.Create(Chart));
      TLineSeries(Chart.Series[1]).LinePen.Width := 2;
      Chart.Series[0].Title := 'Eigenvalue' ;
      Chart.Series[1].Title := 'Cumulative value' ;
      CumulativeSum(MtxPCA.TotalVarPct,tmpVec1);
      DrawValues(MtxPCA.TotalVarPct,Chart.Series[0]);
      DrawValues(tmpVec1,Chart.Series[1]);
    finally
      tmpVec1.Destroy;
    end;
  end;
end;

procedure TfrmPCAWiz.BarlettReport;
var NumDim,i: Integer;
    SignifVec: TVec;
    LineText : String;
begin
  RichEdit.SelAttributes.Style := [fsBold,fsUnderline];
  RichEdit.Lines.Add('Barlett test for dimensionality of data');
  { Barlett test for dimensionality }
  SignifVec := TVec.Create;
  try
    BarlettTest(MtxPCA.Data,NumDim,SignifVec,MtxPCA.PCAMode); { Alpha = 5 % }
    RichEdit.Lines.Add('Barlett test (Alpha = 0.05 ) = '+ IntToStr(NumDim));
    RichEdit.Paragraph.TabCount := 2;
    SetupTabs(RichEdit.Paragraph.TabCount, RichEdit);
    RichEdit.SelAttributes.Color := clBlue;
    RichEdit.Lines.Add('Variable'+chr(9)+'Signif. probability');
    for i := 0 to SignifVec.Length - 1 do
    begin
      LineText := IntToStr(i)+chr(9)+FormatFloat(FormatString,SignifVec.Values[i]);
      RichEdit.Lines.Add(LineText);
    end;
    RichEdit.Lines.Add(Chr(13));
  finally
    SignifVec.Destroy;
  end;
end;

end.
